package uk.ac.cf.spring.demo.sports.security;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.provisioning.InMemoryUserDetailsManager;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import static org.springframework.security.config.Customizer.withDefaults;

@Configuration
@EnableWebSecurity
public class SecurityConfig {

    public static final String[] ENDPOINTS_WHITELIST = {
            "/pics/**",
            "/css/**",          // .CSS Style Files
            "/js/**",           // .JS Files
            "/images/**",       // images
            "/",                // root path
            "/match",           // match data path
            "/match/**",
            "/users",           // user data path
            "/users/**",
            "/api/**",          // API
            "/html/**",         // static HTML
            "/html/register.html",  // registration path
            "/html/login.html",  // login
            "/html/table-tennisrules.html",
            "/rankings",          //ranking data path
            "/rankings/**",
            "/html/table-tennisrules.html"
    };

    public static final String[] ADMIN_WHITELIST = {
            "/admin/**",
    };

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http
                // Configure access rights for paths
                .authorizeHttpRequests(request -> request
                        .requestMatchers(ENDPOINTS_WHITELIST).permitAll() // Allow whitelisted pages without authentication
                        .requestMatchers("/users/current").permitAll()
                        .requestMatchers("/html/backGroundMatch.html").hasRole("ADMIN") // Only users with the ADMIN role are allowed to access
                        .anyRequest().authenticated() // Other paths require authentication
                )

                // Disable CSRF (can be enabled on demand)
                .csrf().disable()

                // Forms Login Configuration
                .formLogin(form -> form
                        .loginPage("/html/login.html").permitAll() // Using a customized login page
                        .defaultSuccessUrl("/html/matchSchedule.html", true) // After successful login, you will be redirected to this page
                        .failureUrl("/html/login.html?error=true") // You will be redirected to this page after a failed login
                )

                // Basic authentication configuration (for API access)
                .httpBasic(withDefaults()) // Basic Auth is supported for API calls

                // Logout Configuration
                .logout(logout -> logout
                        .logoutUrl("/logout") // Set the logout path
                        .logoutSuccessUrl("/html/login.html")  // The path to which you are redirected after the logout is successful
                        .invalidateHttpSession(true)          // Invalidate the session
                        .clearAuthentication(true)            // Clear authentication information
                );
        return http.build();
    }



    @Bean
    public UserDetailsService userDetailsService() {
        UserDetails user =
                User.withDefaultPasswordEncoder()
                        .username("user")
                        .password("password")
                        .roles("USER")
                        .build();
        return new InMemoryUserDetailsManager(user);
    }

    @Bean
    public BCryptPasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();     // Use BCrypt for password encryption
    }
}

